precision highp float;
precision highp int;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform float u_TextureParams1[5];
uniform vec2 u_StageSize;

vec2 adjust(vec2 uv) {
    return vec2(uv.x, 1.0-uv.y);
}

vec2 position(vec2 uv, vec4 slot) {
    return uv+vec2(-slot.x, slot.y)/slot.zw;
}

vec2 position(vec2 uv, vec4 slot, vec4 parent) {
    return (uv*parent.zw-vec2(slot.x, parent.w-slot.y-slot.w))/slot.zw;
}

vec2 position(vec2 uv, vec4 slot, vec4 parent, vec2 anchor) {
    return position(uv, vec4(slot.xy-anchor, slot.zw), parent);
}

vec2 rotate(vec2 src, float angle, vec2 ratio, vec2 anchor) {
    src -= vec2(anchor.x, 1.0-anchor.y);
    return (src*ratio*mat2(cos(angle), -sin(angle), sin(angle), cos(angle)))*(1.0/ratio)+vec2(anchor.x, 1.0-anchor.y);
}

vec2 rotate(vec2 src, float angle, vec2 ratio) {
    return rotate(src, angle, ratio, vec2(0.5));
}

vec2 rotate(vec2 src, float angle) {
    return rotate(src, angle, vec2(1.0, 1.0));
}

bool inside(vec2 pos) {
    return pos.x >= 0.0 && pos.x <= 1.0 && pos.y >= 0.0 && pos.y <= 1.0;
}

bool inside(vec2 pos, float blur) {
    return pos.x >= 0.0 - blur && pos.x <= 1.0 + blur && pos.y >= 0.0 - blur && pos.y <= 1.0 + blur;
}

void blend(in vec4 src, inout vec4 res, float opacity) {
    res = vec4(src.rgb*opacity*(1.-step(src.a, 0.))+ (1.-src.a*opacity)*res.rgb, max(res.a, src.a*opacity));
}
void blend(in vec4 src, inout vec4 res) {
    blend(src, res, 1.);
}

float linear(float t, float b, float c, float d) {
    return mix(b, c, clamp(t/d, 0.0, 1.0));
}

vec2 scale(vec2 uv, vec2 anchor, vec2 scale) {
    return (uv-anchor)/scale  + anchor;
}

vec2 scale(vec2 uv, vec2 anchor, float scale) {
    return (uv-anchor)/scale  + anchor;
}

vec2 transform(vec2 uv, float params[5], vec2 ratio) {
    vec2 pos = uv;
    pos.xy -= (vec2(params[2], .5) - vec2(.5, params[3]));
    pos = scale(pos, vec2(.5), vec2(params[0], params[1]));
    pos = rotate(pos, radians(params[4]), ratio*vec2(params[0], params[1]));
    return pos;
}

vec4 texture2D(sampler2D tex, vec2 uv, float params[5], vec4 base) {
    vec2 ratio = u_StageSize;
    vec2 pos = adjust(uv);
    pos = transform(pos, params, ratio);
    float pct = step(0., pos.x)*step(0., pos.y)*step(0., 1.-pos.x)*step(0., 1.-pos.y);
    return mix(base, texture2D(tex, adjust(pos)), pct);
}

void main(){
    vec2 uv = vec2(textureCoordinate.x, 1.0 - textureCoordinate.y);

    vec4 color = vec4(0.);
    vec4 pic = texture2D(inputImageTexture, adjust(uv), u_TextureParams1, color);
    blend(pic, color, 1.);

    gl_FragColor = color;
}
